package org.junitdoc.core.decisiontable;

import java.util.List;

import org.eclipse.jdt.core.dom.MarkerAnnotation;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.junitdoc.core.jdt.Method;
import org.junitdoc.preferences.PreferenceConstants;
import org.junitdoc.preferences.PreferenceUtils;

public class TestMethod {

	private DecisionTableModel decisionTable;

	private Method method;

	private String name;

	private int index;

	public TestMethod(DecisionTableModel decisionTable,
			MethodDeclaration methodDeclaration, int index) {
		this.decisionTable = decisionTable;
		this.method = new Method(methodDeclaration);
		this.name = methodDeclaration.getName().getFullyQualifiedName();
		this.index = index;
	}

	public DecisionTableModel getDecisionTable() {
		return decisionTable;
	}

	public MethodDeclaration getMethodDeclaration() {
		return method.getSource();
	}

	public String getName() {
		return name;
	}

	public int getIndex() {
		return index;
	}

	public String getCheckState(Type type) {
		if (isChecked(type)) {
			return PreferenceUtils
					.getString(PreferenceConstants.DECISION_TABLE_CHECK_LABEL);
		}
		return PreferenceUtils
				.getString(PreferenceConstants.DECISION_TABLE_UNCHECK_LABEL);
	}

	public boolean isChecked(Type type) {
		return method.hasMarkerAnnotation(type.getFullName());
	}

	public List<MarkerAnnotation> getMarkerAnnotations() {
		return method.getMarkerAnnotations();
	}

	public MarkerAnnotation getMarkerAnnotation(Type type) {
		return method.getMarkerAnnotation(type.getFullName());
	}

	public void selectAndReveal() {
		method.selectAndReveal();
	}
}
